import fs from 'fs'

import { BaseEvent } from '#application/base/baseEvent'
import Storage from '#application/storage'
import { UUID } from '#application/types'
import MapObjectRepository from '#repositories/mapObjectRepository'

interface IPayload {
  mapObjectId: UUID
}

export default class MapObjectRemoveEvent extends BaseEvent {
  public listen(): void {
    this.socket.on('gm:mapObject:remove', this.handleEvent.bind(this))
  }

  private async handleEvent(data: IPayload, callback: (response: boolean) => void): Promise<void> {
    if (!(await this.isCharacterGM())) return

    try {
      // remove the tile from the disk
      const finalFilePath = Storage.getPublicPath('map_objects', data.mapObjectId + '.png')
      fs.unlink(finalFilePath, async (err) => {
        if (err) {
          this.logger.error(`Error deleting object ${data.mapObjectId}: ${err.message}`)
          callback(false)
          return
        }

        await (await MapObjectRepository.getById(data.mapObjectId))?.delete()

        return callback(true)
      })
    } catch (error) {
      this.logger.error(`Error deleting object ${data.mapObjectId}: ${error instanceof Error ? error.message : String(error)}`)
      return callback(false)
    }
  }
}
